      SUBROUTINE QSPDIV (APCORE, MODL, U, VS, INCVS, INCF, INCS, NCOMP,
     *   NVIS, NF, NS)
C-----------------------------------------------------------------------
C! Pseudo AP routine: Divide Gaussian model  visibility into uv data.
C# AP-appl UV
C-----------------------------------------------------------------------
C;  Copyright (C) 1995, 2000, 2006, 2008, 2012, 2019
C;  Associated Universities, Inc. Washington DC, USA.
C;
C;  This program is free software; you can redistribute it and/or
C;  modify it under the terms of the GNU General Public License as
C;  published by the Free Software Foundation; either version 2 of
C;  the License, or (at your option) any later version.
C;
C;  This program is distributed in the hope that it will be useful,
C;  but WITHOUT ANY WARRANTY; without even the implied warranty of
C;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C;  GNU General Public License for more details.
C;
C;  You should have received a copy of the GNU General Public
C;  License along with this program; if not, write to the Free
C;  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge,
C;  MA 02139, USA.
C;
C;  Correspondence concerning AIPS should be addressed as follows:
C;         Internet email: aipsmail@nrao.edu.
C;         Postal address: AIPS Project Office
C;                         National Radio Astronomy Observatory
C;                         520 Edgemont Road
C;                         Charlottesville, VA 22903-2475 USA
C-----------------------------------------------------------------------
C   Vector compiler version
C   Divides the model visibility derived from Gaussian components into
C   visibility records.  Arbitrary numbers of frequencies and
C   polarizations can be processed.  The weights of the visibilities
C   are multiplied times  the amplitude of the model visibility.
C   Inputs:
C      MODL  I    Base address of Gaussian components, increment = 6
C                 0 = 3 * Amplitude
C                 1 = -2 * PI * X (radians)
C                 2 = -2 * PI * Y
C                 3 = -2 * PI * Z
C                 4 = R = radius * 3.0462740E-5
C                 5 = MIN = min arg (0.1 good)
C      U     I    Base address of U, assumed followed by V, W
C      VS    I    Base address of vis rec. (real, imag, wt)
C      INCVS I    Increment of VS for next visibility
C      INCF  I    Increment of VS for next frequency
C      INCS  I    Increment of VS for next Stokes (RR or LL)
C      NCOMP I    Number of Gaussian components.
C      NVIS  I    Number of visibilities.
C      NF    I    Number of frequencies.
C      NS    I    Number of Stokes (usually 1 or 2)
C    Also uses AP locations 0 and 1 and expects an array
C    beginning in location 2 should be an array of length NF :
C             Freq(0) / Freq(ref) - 1.0
C             Freq(1) / Freq(ref)   - 1.0
C                     .
C                     .
C                     .
C            Freq(NF-1) / Freq(ref) - 1.0
C   Note: all addresses are 0 relative and needed to be incremented by
C   1 to work in Fortran.
C-----------------------------------------------------------------------
      DOUBLE PRECISION APCORE(*)
      INTEGER   MODL, U, VS, INCVS, INCF, INCS, NCOMP, NVIS, NF, NS

      INTEGER   X, JVS, IVS, IF, IV, IS, ICOMP, AMP, R, IMIN
      LONGINT   IU, F, JX, JR, JMIN, JAMP, JA
      DOUBLE PRECISION FREQF, SUMRE, SUMIM, REMOD, IMMOD, WT, TEMP, R2,
     *   PHS, ARGV, AMPL
      INCLUDE 'INCS:DAPC.INC'
C-----------------------------------------------------------------------
C                                        Make sure that there is data.
      IF ((NCOMP.LE.0) .OR. (NVIS.LE.0) .OR. (NF.LE.0) .OR.
     *   (NS.LE.0))  GO TO 999
C                                       Setup array addresses
      X = MODL + 1
      R = MODL + 4
      AMP = MODL
      IMIN = MODL + 5
      IU = U + PSAPOF - 1
      IVS = VS
C                                       Begin visibility loop
C$OMP PARALLEL DO PRIVATE (IV, JVS, F, IF, SUMRE, SUMIM, JX, JAMP, JR,
C$OMP+               FREQF, ICOMP, PHS, REMOD, IMMOD, IS, R2, JMIN, WT,
C$OMP+               TEMP, JA, IVS, IU, ARGV, AMPL), SHARE(APCORE)
      DO 300 IV = 1,NVIS
C                                        Get ready for freq. loop
         JVS = IVS
         F = 1 + PSAPOF
         R2 = SQRT (APCORE(IU+1)*APCORE(IU+1) +
     *      APCORE(IU+2)*APCORE(IU+2))
C                                        Begin frequency loop
         DO 200 IF = 1,NF
            IF (ABS (APCORE(F+1)).GT.1.0E-20) THEN
               FREQF = 1.0D0 + APCORE(F+1)
            ELSE
               FREQF = 1.0D0
               END IF
C                                       Loop over component
            SUMRE = 0.0D0
            SUMIM = 0.0D0
            JX = X + PSAPOF
            JR = R + PSAPOF
            JAMP = AMP + PSAPOF
            JMIN = IMIN + PSAPOF
            DO 100 ICOMP = 1,NCOMP
               PHS = FREQF * (APCORE(JX) * APCORE(IU+1)
     *                    + APCORE(JX+1) * APCORE(IU+2)
     *                    + APCORE(JX+2) * APCORE(IU+3))
               ARGV = FREQF * R2 * APCORE(JR)
               ARGV = MAX (ARGV, APCORE(JMIN))
               AMPL = APCORE(JAMP) * ((SIN (ARGV) / ARGV**3) -
     *            (COS (ARGV) / ARGV**2))
               SUMRE = SUMRE + AMPL * COS(PHS)
               SUMIM = SUMIM + AMPL * SIN(PHS)
               JX = JX + 6
               JR = JR + 6
               JAMP = JAMP + 6
               JMIN = JMIN + 6
 100           CONTINUE
C                                       Correct visibility
            JA = JVS + PSAPOF
C                                        Setup
            WT = (SUMRE * SUMRE + SUMIM * SUMIM)
            WT = MAX (1.0D-15, WT)
            REMOD = SUMRE / WT
            IMMOD = SUMIM / WT
C                                       Loop over Stokes (1 or 2,
C                                       no point in vectorizing)
            DO 150 IS = 1,NS
C                                       Divide model
               TEMP = REMOD * APCORE(JA) + IMMOD * APCORE(JA+1)
               APCORE(JA+1) = REMOD * APCORE(JA+1) - IMMOD * APCORE(JA)
               APCORE(JA) = TEMP
C                                       Modify weight by amp squared
               APCORE(JA+2) = APCORE(JA+2) * WT
               JA = JA + INCS
 150           CONTINUE
C                                        Update vis pointer
            JVS = JVS + INCF
            F = F + 1
 200        CONTINUE
C                                        Update pointers
         IVS = IVS + INCVS
         IU = IU + INCVS
 300     CONTINUE
C$OMP END PARALLEL DO
C
 999  RETURN
      END
